## vim: set syn=on ts=4 sw=4 sts=0 noet foldmethod=indent:
## purpose: functions to parse status.dat for monitoring-proxy.
## copyright: B1 Systems GmbH <info@b1-systems.de>, 2009.
## license: GPLv3+, http://www.gnu.org/licenses/gpl-3.0.html
## author: Christian Schneemann <schneemann@b1-systems.de>, 2009.


our @stat_dat;
sub read_stat_dat
{
	our @stat_dat;
	if (!@stat_dat[0])
	{
		our $status_dat;
		open(STAT_DAT, "<$status_dat") || die $!;
			@stat_dat = <STAT_DAT>;
		close STAT_DAT;
	}
	return @stat_dat;
}

sub get_hosts #returns array of _all_ hosts
{
	my @stat_dat = &read_stat_dat();
	my @hosts;
	for( my $i = 0; $i < @stat_dat; $i++)
	{
		if ($stat_dat[$i] =~ /^hoststatus\s+\{$/)
		{
			while ($stat_dat[$i] !~ /\}$/)
			{
				if ($stat_dat[$i] =~ /host_name=(.*)$/)
				{
					$host_name = $1;
				}
			$i++;
			}
			my $comments = &count_comments_for_host($host_name);
			my @services = &count_services_for_host($host_name);
			push (@hosts, { host_name => $host_name,
                   comments => $comments,
                   services => $services[0],
                   services_warn => $services[1],
                   services_crit => $services[2],
                   services_unkn => $services[3]} );
		}
	}
	@stat_dat = ();
	return @hosts;
}

sub get_info_for_host #returns hash full of host information
{
	my @stat_dat = &read_stat_dat();
	my $found = 0;
	my %host_data;
	for (my $i=0; $i<@stat_dat; $i++)
	{
		if ($stat_dat[$i] =~ /^hoststatus\s+\{$/)
		{
			while ($stat_dat[$i] !~ /\}/)
			{
				if ($stat_dat[$i] =~ /host_name=$_[0]$/) {
					$found = 1;
					$host_data{'host_name'} = $_[0];
				}
				if ($found && $stat_dat[$i] =~ /\tcurrent_state=(.*)$/) {
					$host_data{'current_state'} = $1;
				}
				if ($found && $stat_dat[$i] =~ /\tplugin_output=(.*)$/) {
					warn("plugin_output: $1");
					$host_data{'plugin_output'} = $1;
				}
				if ($found && $stat_dat[$i] =~ /\tlong_plugin_output=(.*)$/) {
					$host_data{'long_plugin_output'} = $1;
				}
				if ($found && $stat_dat[$i] =~ /\tperformance_data=(.*)$/) {
					$host_data{'performance_data'} = $1;
				}
				if ($found && $stat_dat[$i] =~ /\tproblem_has_been_acknowledged=(.*)$/) {
					$host_data{'problem_has_been_acknowledged'} = $1;
				}
#				if ($found && $stat_dat[$i] =~ /current_problem_id=(.*)$/) {
#					$host_data{'current_problem_id'} = $1;
#				}
				$i++;
			}
			if ($found == 1)
			{
				$i = @stat_dat;
			}
		}
	}#FIXME services werden nicht eingetragen
	my @services = &count_services_for_host($_[0]);
	$host_data{'services'} = $services[0];
	$host_data{'services_warn'} = $services[1];
	$host_data{'services_crit'} = $services[2];
	$host_data{'services_unkn'} = $services[3];	
	if ($found != 1)
	{
		 $host_data{'error'} = "not found";
	}
	%{$host_data{'comments'}} = &get_comments_for_host($_[0]);
	return %host_data;
}

sub get_comments_for_host()
{
	my @stat_dat = &read_stat_dat();
	my $found;
	$comments{'host_name'} = $_[0];
	my (%comments, $comment_id, $entry_time, $author, $comment_data);
	for (my $i=0; $i<@stat_dat; $i++)
	{
		if ($stat_dat[$i] =~ /^hostcomment\s+\{$/)
		{
			while ($stat_dat[$i] !~ /\}/)
			{
				if ($stat_dat[$i] =~ /host_name=$_[0]$/) {
					$found = 1;
				}
				elsif ($found && $stat_dat[$i] =~ /comment_id=(.*)$/) {
					$comment_id = $1;
				}
				elsif ($found && $stat_dat[$i] =~ /entry_time=(.*)$/) {
					$entry_time = $1;
				}
				elsif ($found && $stat_dat[$i] =~ /author=(.*)$/) {
					$author = $1;
				}
				elsif ($found && $stat_dat[$i] =~ /comment_data=(.*)$/) {
					$comment_data = $1;
				}
				$i++;
			}
			if ($comment_id)
			{
				push(@{$comments{'comment'}}, {comment_id => $comment_id,
                      entry_time => $entry_time,
                      author => $author,
                      comment_data => $comment_data,
                      });
			}
		}
	}
	return %comments;
}

sub count_comments_for_host()
{
	my @stat_dat = &read_stat_dat();
	my $found = 0;
	for (my $i=0; $i<@stat_dat; $i++)
	{
		if ($stat_dat[$i] =~ /^hostcomment\s+\{$/)
		{
			$i++;
			while ($stat_dat[$i] !~ /\}/)
			{
				if ($stat_dat[$i] =~ /host_name=$_[0]$/) {
					$found++;
				}
				$i++;
			}
		}
	}
	return $found;
}

sub count_services_for_host()
{
	my @stat_dat = &read_stat_dat();
	my $found=0;
	my $warning = 0;
	my $critical = 0;
	my $unknown = 0;

	for (my $i=0; $i<@stat_dat; $i++)
	{
		if ($stat_dat[$i] =~ /^servicestatus\s+\{$/)
		{
			$i++;
			while ($stat_dat[$i] !~ /\}/)
			{
				if ($stat_dat[$i] =~ /host_name=$_[0]$/) {
					$found++;
				}elsif ($stat_dat[$i] =~ /current_state=(.*)$/) {
					if ($1 != 0)
					{
						if ($1 == 1) 
						{
							$warning++; 
						}elsif ($1 == 2) 
						{
							$critical++; 
						}elsif ($1 == 3) 
						{
							$unknown++; 
						}
					}
				}
				$i++; 
			}
		}
	}
	return ($found, $warning, $critical, $unknown);
}

sub get_services_for_host #returns hash of services for given host
{
	my $search_for_host;
	if ($_[0])
	{
		$search_for_host = $_[0];
	} else
	{
		die("ERROR in get_services_for_host: no host given"); #FIXME die austauschen.. meldung muss erscheinen!
	}
	my @stat_dat = &read_stat_dat();
	my %services;
	my ($host_name, $service_description, $current_state, $problem_has_been_acknowledged);
	my $plugin_output;
	for( my $i = 0; $i < @stat_dat; $i++)
	{
		if ($stat_dat[$i] =~ /^servicestatus\s+\{$/)
		{
			my $found = 0;
			while ($stat_dat[$i] !~ /\}$/)
			{
				if ($stat_dat[$i] =~ /host_name=$search_for_host$/)
				{
					$host_name = $search_for_host;
					$found = 1;
				}
				if ($found && $stat_dat[$i] =~ /service_description=(.*)$/)
				{
					$service_description = $1;
				}
				if ($found && $stat_dat[$i] =~ /current_state=(.*)$/)
				{
					$current_state= $1;
				}
				if ($found && $stat_dat[$i] =~ /\tplugin_output=(.*)$/)
				{
					$plugin_output = $1;
				}
				if ($found && $stat_dat[$i] =~ /problem_has_been_acknowledged=(.*)$/)
				{
					$problem_has_been_acknowledged = $1;
				}
				$i++;
			}	
			if ($found) {
				push(@{$services{'service'}}, {service_description => $service_description,
						current_state => $current_state,
						plugin_output => $plugin_output,
						problem_has_been_acknowledged => $problem_has_been_acknowledged,
						});
			}
		}
	}
	@stat_dat= ();
	$services{'host_name'} = $host_name;
	return %services;
}

sub get_service()
{
	my $search_for_host = $_[0];
	my $search_for_srv = $_[1];
	my @stat_dat = &read_stat_dat();
    my %srv;
	$srv{'host_name'} = "";
	
	my ($found, $found_host, $found_srv);
	for( my $i = 0; $i < @stat_dat; $i++)
    { 
		$found = 0;
        if ($stat_dat[$i] =~ /^servicestatus\s+\{$/)
		{
            my $found_host = 0;
			my $found_srv = 0;
            while ($stat_dat[$i] !~ /\}$/)
			{
				if ($found_host == 1 && $found_srv == 1)
				{
					$found = 1;
				}
                if ($stat_dat[$i] =~ /host_name=$search_for_host$/)
            	{
                    $found_host = 1;
					$srv{'host_name'} = $search_for_host;
				}
				if ($stat_dat[$i] =~ /service_description=$search_for_srv/)
				{
					$found_srv = 1;
					$srv{'service_description'} = $search_for_srv;
            	}
            	if ($found && $stat_dat[$i] =~ /current_state=(.*)$/)
            	{
            	    $srv{'current_state'} = $1;
            	}
            	if ($found && $stat_dat[$i] =~ /\tplugin_output=(.*)$/)
            	{
            	    $srv{'plugin_output'} =  $1;
            	}
				if ($found && $stat_dat[$i] =~ /\tlong_plugin_output=(.*)$/)
				{
					if ($1 != "")
					{ $srv{'long_plugin_output'} = $1; }
				}
				if ($found && $stat_dat[$i] =~ /\tperformance_data=(.*)$/)
				{
					$srv{'performance_data'} = $1;
				}
            	$i++;
        	}
        }
		if ($found == 1)
		{
			$i = @stat_dat;
		}
	}
	if ($found == 0 )
	{
		$srv{'error'} = "not found";
	}
	return %srv;
}
1
