## vim: set syn=on ts=4 sw=4 sts=0 noet foldmethod=indent:
## purpose: general functions for monitoring-proxy.
## copyright: B1 Systems GmbH <info@b1-systems.de>, 2009.
## license: GPLv3+, http://www.gnu.org/licenses/gpl-3.0.html
## author: Christian Schneemann <schneemann@b1-systems.de>, 2009.

sub parse_config() # parse the configuration file given in $_[0]
{
	my %config;
	open (CF, "<$_[0]");
	while (<CF>)
	{
		if ( /^#/)
		{
			next;
		}
		if (/ext_command_file\s+"(.+)";\s*$/)
		{
			$config{'ext_command_file'} = $1;
			next;
		}
		if (/status_dat\s+"(.+)";\s*$/)
		{
			$config{'status_dat'} = $1;
			next;
		}
		if (/is_readonly\s+([0,1]);\s*$/)
		{
			$config{'is_readonly'} = $1;
			next;
		}
		if (/datasource\s+"(.*)";\s*$/)
		{
			if ($1 eq "file")
			{
				$config{'datasource'} = "file";
			} elsif ($1 eq "mysql")
			{
				die ("not implemented yet!\n");
			#if mysql read username/password etc needed for database connection
			}
			next;
		}
	}
	close(CF);

	my $fault = 0;
	if ($config{'ext_command_file'} eq "")
	{
		warn("ext_command_file not set!\n");
		$fault = 1;
	}
	if ($config{'status_dat'} eq "")
	{
		warn ("status_dat not set!\n");
		$fault = 1;
	}
	if ($config{'is_readonly'} eq "")
	{
		warn("is_readonly not set!\n");
		$fault = 1;
	}
	if ($config{'datasource'} ne "")
	{
		if ($config{'datasource'} eq "file")
		{
			#do nothing..
		} elsif ($config{'datasource'} eq "mysql")
		{
			#test host, user, password, db, table...
		} else {
			warn ("datasource set to unkown!\n");
			$fault = 1;
		}
	}  else {
		warn ("source: \"$config{'datasource'}\"");
		warn ("datasource not set!\n");
		$fault = 1;
	}

	if ($fault)
	{
		die ("please review your configuration file!\n");
	}

	return %config;
}

sub get_variables #(host_name, service_description, sticky, notify, persistent, author, comment)
{
	if ($ENV{'HTTP_USER_AGENT'})
	{
		our $q;
		my %needed_values;
		for ( my $i = 0; $i < @_; $i++)
		{
			$needed_values{$_[$i]} = $q->param($_[$i]);
		}
		return %needed_values;
	}
}

sub valid_function
{
	our $is_readonly;
	# possible values for {'submit'} => 1 = command writes to external commands file
	#                                   2 = command just reads from status.dat
	my @possible_functions = ( { name => "ACKNOWLEDGE_HOST_PROBLEM", submit => 1, },
                               { name => "ACKNOWLEDGE_SVC_PROBLEM", submit => 1, },
                               { name => "DEL_ALL_HOST_COMMENTS", submit => 1, },
                               { name => "DEL_ALL_SVC_COMMENTS", submit => 1, },
                               { name => "ADD_SVC_COMMENT", submit => 1, },
                               { name => "ADD_HOST_COMMENT", submit => 1, },
                               { name => "DISABLE_NOTIFICATIONS", submit => 1, },
#                               { name => "FORCE_SVC_RESCHEDULE", submit => 1, }, #to be implemented FIXME
                               { name => "SHOW_HOST", submit => 2, },
                               { name => "SHOW_SERVICE", submit => 2, }, 
                               { name => "LIST_HOSTS", submit => 2, },
                               { name => "LIST_SERVICES", submit => 2, }, );

	for (my $i=0; $i<@possible_functions; $i++)
	{
		if ( $_[0] eq $possible_functions[$i]{'name'} )
		{
			if ($is_readonly == 1 &&  $possible_functions[$i]{'submit'} == 1)
			{
				return 0;
			} else {
				return $possible_functions[$i]{'submit'};
			}
		}
	}
	return 0;
}

sub acknowledge_host_problem()
{
	my @needed_values = ("host", "sticky", "notify", "persistent", "author", "comment");
	my %v = get_variables(@needed_values);
	my $tstamp = time();
	return &submit_command("[$tstamp] ACKNOWLEDGE_HOST_PROBLEM;$v{host};$v{sticky};$v{notify};$v{persistent};$v{author};$v{comment}\n");
}

sub acknowledge_svc_problem()
{
	my @needed_values = ( "host", "service_description", "sticky", "notify", "persistent", "author", "comment" );
	my $tstamp = time();
	my %v = get_variables(@needed_values);
	return &submit_command("[$tstamp] ACKNOWLEDGE_SVC_PROBLEM;$v{'host'};$v{'service_description'};$v{'sticky'};$v{'notify'};$v{'persistent'};$v{'author'};$v{'comment'}\n");
}

sub del_all_host_comments()
{
	my @needed_values = ("host");
	my $tstamp = time();
	my %v = get_variables(@needed_values);
	return &submit_command("[$tstamp] DEL_ALL_HOST_COMMENTS;$v{'host'}\n");
}

sub del_host_comment()
{
	my @needed_values = ("comment_id");
	my $tstamp = time();
	my %v = get_variables(@needed_values);
	return &submit_command("[$tstamp] DEL_HOST_COMMENT;$v{'comment_id'}\n");
}

sub del_all_svc_comments()
{
	my @needed_values = ("host", "service_description");
	my %v = get_variables(@needed_values);
	my $tstamp = time();
	return &submit_command("[$tstamp] DEL_ALL_SVC_COMMENTS;$v{'host'};$v{'servcie_description'}\n");
}

sub del_svc_comment()
{
	my @needed_values = ("comment_id");
	my %v = get_variables(@needed_values);
	return &submit_command("[$tstamp] DEL_SVC_COMMENT;$v{'comment_id'}\n");
}

sub add_svc_comment()
{
	my @needed_values = ("host", "service_description", "persistent", "author", "comment");
	my %v = get_variables(@needed_values);
	my $tstamp = time();
	return &submit_command("[$tstamp] ADD_SVC_COMMENT;$v{'host'};$v{'service_description'};$v{'persisten'};$v{'author'};$v{'comment'}\n");
}

sub add_host_comment()
{
	my %v = get_variables("host", "persistent", "author", "comment");
	my $tstamp = time();
	return &submit_command("[$tstamp] ADD_HOST_COMMENT;$v{'host'};$v{'persistent'};$v{'author'};$v{'comment'}\n");
}

sub disable_notification()
{
	my $tstamp = time();
	return &submit_command("[$tstamp] DISABLE_NOTIFICATION;\n");
}

sub show_host()
{
	my %v = &get_variables("host");
	my %host_data = &get_info_for_host($v{'host'});
	return &xml_out(\%host_data, "show_host");
}

sub show_service()
{
	my %v = &get_variables("host", "service_description");
	my %srv_data = &get_service($v{'host'}, $v{'service_description'});
	return &xml_out(\%srv_data, "show_service");
}

sub list_services()
{
	my %v = &get_variables("host");
	my %services = &get_services_for_host($v{"host"});
	return &xml_out(\%services, "list_services");
}

sub list_hosts()
{
	my @hosts = &get_hosts();
	return &xml_out(\@hosts, "list_hosts");
}
sub xml_out()
{
	my $data = $_[0];
	my $function = $_[1];
	my $return = "<?xml version=\"1.0\" ?>\n";
	if ($function eq "list_hosts") # build xml output for LIST_HOSTS
	{
		$return .= "<hosts>";
		foreach $host (@$data)
		{
			$return .= "<host>";
				$return .= "<host_name>";
						$return .= $host->{'host_name'};
				$return .= "</host_name>";
				$return .= "<comments>";
					$return .= $host->{'comments'};
				$return .= "</comments>";
				$return .= "<services>";
					$return .= $host->{'services'};
				$return .= "</services>";
				$return .= "<services_warn>";
					$return .= $host->{'services_warn'};
				$return .= "</services_warn>";
				$return .= "<services_crit>";
					$return .= $host->{'services_crit'};
				$return .= "</services_crit>";
				$return .= "<services_unkn>";
					$return .= $host->{'services_unkn'};
				$return .= "</services_unkn>";
			$return .= "</host>";
		}
		$return .= "</hosts>";
	}
	elsif ($function eq "show_host") # build xml output for SHOW_HOST
	{
		if(defined($data->{'error'}))
		{
			$return .= "<error>";
				$return .= "$data->{'error'}";
			$return .= "</error>";
		} else
		{
			$return .= "<hostinfo>";
				$return .= "<error>";
				if ($data->{'error'} ne "")
				{
					$return .= $data->{'error'};
				} else {
					$return .= "";
				}
				$return .= "</error>";
				$return .= "<host_name>";
					$return .= $data->{'host_name'};
				$return .= "</host_name>";
				$return .= "<current_state>";
					$return .= $data->{'current_state'};
				$return .= "</current_state>";
				$return .= "<plugin_output>";
					$return .= $data->{'plugin_output'};
				$return .= "</plugin_output>";
				$return .= "<long_plugin_output>";
					$return .= $data->{'long_plugin_output'};
				$return .= "</long_plugin_output>";
				$return .= "<performance_data>";
					$return .= $data->{'performance_data'};
				$return .= "</performance_data>";
				$return .= "<problem_has_been_acknowledged>";
					$return .= $data->{'problem_has_been_acknowledged'};
				$return .= "</problem_has_been_acknowledged>";
				$return .= "<services>";
					$return .= "<service_nrs>";
						$return .= $data->{'services'};
					$return .= "</service_nrs>";
					$return .= "<services_warn>";
						$return .= $->{'services_warn'};
					$return .= "</services_warn>";
					$return .= "<services_crit>";
						$return .= $data->{'services_crit'};
					$return .= "</services_crit>";
					$return .= "<services_unkn>";
						$return .= $data->{'services_unkn'};
					$return .= "</services_unkn>";
				$return .= "</services>";
				$return .= "<comments>";
					foreach $comment (@{$data->{'comments'}{'comment'}})
					{
						$return .= "<comment>";
							$return .= "<comment_id>";
								$return .= $comment->{'comment_id'};
							$return .= "</comment_id>";
							$return .= "<entry_time>";
								$return .= $comment->{'entry_time'};
							$return .= "</entry_time>";
							$return .= "<author>";
								$return .= $comment->{'author'};
							$return .= "</author>";
							$return .= "<comment_data>";
								$return .= $comment->{'comment_data'};
							$return .= "</comment_data>";
						$return .= "</comment>";
					}
				$return .= "</comments>";
			$return .= "</hostinfo>";
		}
	}
	elsif ($function eq "show_service")
	{
		$return .= "<service>";
			$return .= "<error>";
			if ($data->{'error'} ne "")
			{
				$return .= $data->{'error'};
			}
			else
			{
				$return .= "";
			}
			$return .= "</error>";
			$return .= "<host_name>";
				$return .= $data->{'host_name'};
			$return .= "</host_name>";
			$return .= "<service_description>";
				$return .= $data->{'service_description'};
			$return .= "</service_description>";
			$return .= "<current_state>";
				$return .= $data->{'current_state'};
			$return .= "</current_state>";
			$return .= "<plugin_output>";
				$return .= $data->{'plugin_output'};
			$return .= "</plugin_output>";
			$return .= "<long_plugin_output>";
				$return .= $data->{'long_plugin_output'};
			$return .= "</long_plugin_output>";
			$return .= "<performance_data>";
				$return .= $data->{'performance_data'};
			$return .= "</performance_data>";
		$return .= "</service>";
	}
	elsif ($function eq "list_services")
	{
		$return .= "<services>";
			$return .= "<host_name>";
				$return .= $data->{'host_name'};
			$return .= "</host_name>";
			foreach $service (@{$data->{'service'}})
			{
				$return .= "<service>";
					$return .= "<service_description>";
						$return .= $service->{'service_description'};
					$return .= "</service_description>";
					$return .= "<plugin_output>";
						$return .= $sercice->{'plugin_output'};
					$return .= "</plugin_output>";
					$return .= "<current_state>";
						$return .= $service->{'current_state'};
					$return .= "</current_state>";
					$return .= "<problem_has_been_acknowledged>";
						$return .= $service->{'problem_has_been_acknowledged'};
					$return .= "</problem_has_been_acknowledged>";
				$return .= "</service>";
			}
		$return .= "</services>";
	}
	return $return;
}
sub submit_command #writes the command into the external commands file
{
	our $ext_command_file;
	if ( -w ($ext_command_file)) 
	{
		open(EC ,">>$ext_command_file") || die "ERROR: $!";
			print EC $_[0]; 
		close EC;
	}else  {
		return "$ext_command_file is not writeable, is nagios running?";
	}
	return "command submitted";
}

1
