#!/usr/bin/perl
## vim: set syn=on ts=4 sw=4 sts=0 noet foldmethod=indent:
## purpose: remote control your monitoring server (nagios/icinga).
## copyright: B1 Systems GmbH <info@b1-systems.de>, 2009.
## license: GPLv3+, http://www.gnu.org/licenses/gpl-3.0.html
## author: Christian Schneemann <schneemann@b1-systems.de>, 2009.

# Version: 0.0.7

use warnings;
use strict;

use lib ".";
BEGIN{ unshift (@INC, ".");}

require "monitoring-proxy.func.pl";

our $config_file = ""; #set path to your confuration file here if 
                      #you don't use a default place
if ($config_file eq "" || !-e $config_file)
{
	#go through of possible files
	if(-e "/etc/nagios/monitoring-proxy.conf")
	{
		$config_file = "/etc/nagios/monitoring-proxy.conf";
	} elsif (-e "/etc/icinga/monitoring-proxy.conf" )
	{
		$config_file = "/etc/icinga/monitoring-proxy.conf";
	} elsif (-e "/etc/monitoring-proxy.conf")
	{
		$config_file = "/etc/monitoring-proxy.conf";
	} else
	{
		die ("no configuration file found!\n");
	}
}

my %config = &parse_config($config_file);

our $ext_command_file = $config{'ext_command_file'};
our $status_dat = $config{'status_dat'};
our $is_readonly = $config{'is_readonly'};
my $datasource = $config{'datasource'};

%config = ();

if ($datasource eq "file")
{
	warn "use file-parser";
	require "file-parser.pl";
} elsif ($datasource eq "mysql")
{
	warn "not implemented yet\n";
}



my @func;
if ($ENV{'HTTP_USER_AGENT'})
{
	use CGI::Simple;
	our $q = new CGI::Simple;
	@func = $q->param('func');
} else {
	# selben funktionsumfang wie oben implementieren, ohne cgi, falls das script über die konsole aufgerufen wird
	# niedrige Prio..
}

if ( @func != 1 || !valid_function($func[0])) #FIXME valid_function so umbauen, dass man in einen read_only modus wechseln kann, keine ext commandos
{
	#FIXME
	# add errorhandling, change to xml layout
	print "Content-type: text/html\n\n";
	print '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">', "\n";
	print "<html><head><title>monitoring-EC-proxy</title></head><body>\n";
	print "function not found!";
	print "</body></html>";
} else {
	if (-e $ext_command_file) #if $ext_command_file does not exist nagios is not running
	{
		my $output;
		my $func = $func[0];
		if ($func eq "ACKNOWLEDGE_HOST_PROBLEM")
		{
			$output = &acknowledge_host_problem();
		} elsif ($func eq "ACKNOWLEDGE_SVC_PROBLEM")
		{
			$output = &acknowledge_svc_problem();
		} elsif ($func eq "SHOW_HOST")
		{
			$output = &show_host();
		} elsif ($func eq "SHOW_SERVICE")
		{
			$output = &show_service();
		} elsif ($func eq "LIST_HOSTS")
		{
			$output = &list_hosts();
		} elsif ($func eq "LIST_SERVICES")
		{
			$output = &list_services();
		} elsif ($func eq "DISABLE_NOTIFICATION")
		{
			$output = &disable_notivication();
		} elsif ($func eq "ADD_HOST_COMMENT")
		{
			$output = &add_host_comment();
		} elsif ($func eq "ADD_SVC_COMMENT")
		{
			$output = &add_svc_comment();
		} elsif ($func eq "DEL_ALL_SVC_COMMENTS")
		{
			$output = &del_all_svc_comments();
		} elsif ($func eq "DEL_ALL_HOST_COMMENTS")
		{
			$output = &del_all_host_comments();
		} elsif ($func eq "DEL_HOST_COMMENT")
		{
			$output = &del_host_comment();
		} elsif ($func eq "DEL_SVC_COMMENT")
		{
			$output = &del_svc_comment();
		}

		if (valid_function($func) == 1) #can be removed, not needed anymore, adjust valid_function()
		{
			#$output = submit_command($command_string);
		}
		print "Content-type: text/html\n\n";
		print $output;
	} else {
		warn "The external command file does not exist, is nagios is running?";
	}
}

